/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.components.switchFabric;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.gasp.Branch;
import com.sunlabs.archsim.gasp.BranchAction;

/**
 * Branch - two outputs, directs an output to them in an alternating fashion.
 * @author ib27688
 *
 */
public class ToggleBranch extends Branch {
	
	public ToggleBranch(String name, Parameters params) {
		super(name, 2, new ToggleAction(2) );
		
		
		int numOut= SwitchFabricDefaults.BRANCH_FANOUT_DEF;
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				
				if (pName.equals(SwitchFabricDefaults.BRANCH_FANOUT)
						|| pName.equals(FleetGlobal.PARAMETER) ) {
					try {
						numOut= Integer.parseInt(sValue);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + SwitchFabricDefaults.BRANCH_FANOUT
								+ ": " + sValue
								+ ". Value is IGNORED and the default value of "
								+ numOut + " is used.");						
					}
				} // else
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				}
				
			} // for
		} // if params != null
				
		
		build(numOut, new ToggleAction(numOut));
		
	}	
} // Toggle

class ToggleAction extends BranchAction {
	static final String name= "ToggleAction";
	
	int nextBranch;
	
	public ToggleAction(int numOut) {
		super(name, numOut);
		nextBranch= 0;
	}
		
	public int selectBranch(Object data) {
		int s= nextBranch;
		nextBranch= (nextBranch+1) % numBranches;
		return s;
	}
} // ToggleAction