/*
 * Created on October 26, 2005, 4:58 PM
 */

package com.sunlabs.archsim.fleet.ships.adder_oob;


import java.util.List;
// import java.util.Vector;
import java.util.Iterator;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;


import com.sunlabs.archsim.fleet.ships.testing.DataSink;

import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import junit.textui.TestRunner; 

/*
 * Copyright ??? 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *
 */

/**
 *
 * @author mh193489
 */
public class TestAdder_oob extends TestCase {
    
    private static String fileName= "src/" 
    		+ TestAdder_oob.class.getPackage().getName().replace('.', '/') 
    		+ "/TestAdderOOBScript.xml";
    
    private static Director director;
    
    /* References to components to be tested */
    private static DataSink dataSink;
    
    private static final String DATA1_SOURCE= "Data1Source";
    private static final String DATA2_SOURCE= "Data2Source";
//    private static final String DATA_SINK= "DataSink";    
    
    protected void setUp() {
        // Reset sources and sinks
        PayloadSource.clear();      
        
        // for sum test
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(2));
        
        // for overflow test
        PayloadSource.add(DATA1_SOURCE, new Data(0));
        PayloadSource.add(DATA2_SOURCE, new Data(AdderDefaults.OVERFLOW));
        
        // for underflow test
        PayloadSource.add(DATA1_SOURCE, new Data(0));
        PayloadSource.add(DATA2_SOURCE, new Data(AdderDefaults.UNDERFLOW));
        
        // for overflow OOB on inputs test
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(AdderDefaults.OVERFLOW, false));
        
        PayloadSource.add(DATA1_SOURCE, new Data(AdderDefaults.OVERFLOW, false));
        PayloadSource.add(DATA2_SOURCE, new Data(2));
        
        // for LAST OOB on inputs
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(2));
        
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(null, false));
        
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA1_SOURCE, new Data(null, false));
        
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(2));

        PayloadSource.add(DATA1_SOURCE, new Data(null, false));
        PayloadSource.add(DATA2_SOURCE, new Data(1));
        
        PayloadSource.add(DATA2_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(null, false));
        
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(2));

        // initialize the tests
        director= Director.getInstance();
        
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(fileName);
        
        Runner r= new Runner();
        
        r.perform(ops);
        
        director.journal("Finished Test run");
        
        // get references to the components
        dataSink = (DataSink)Entities.find("DataSink");
    }
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        director.reset(); 	
    }
    
    public static Test suite() {
        return new TestSuite(TestAdder_oob.class);
    }
    
    public static void main(String[] args) {
 
        // run the tests
        TestRunner.run(TestAdder_oob.class);
        
    } // main
    
    public void testTestSetup() {
        // data sink
        assertNotNull(dataSink);
        assertNotNull(dataSink.receivedData);
        for (Iterator<Data> i= dataSink.receivedData.iterator(); i.hasNext();) {
            // check if data is not null
            Data data= i.next();
            assertNotNull(data);
            // check if data value is not null, in case data is valid
            if (data.valid) {
                assertNotNull(data.value);
            }
        }
        
    } 
 
    
    /* Tests for the data interface */
    
    public void testSum() {
        assertTrue(dataSink.receivedData.get(0).valid);
        assertEquals( ((Integer)dataSink.receivedData.get(0).value).intValue(), 3);
    }
    
    
    public void testOverflow() {
        assertFalse(dataSink.receivedData.get(1).valid);
        assertEquals(dataSink.receivedData.get(1).value, AdderDefaults.PLUS_INFINITY);
    }
    
    
    public void testUnderflow() {
        assertFalse(dataSink.receivedData.get(2).valid);
        assertEquals(dataSink.receivedData.get(2).value, AdderDefaults.MINUS_INFINITY);
    }
    
    public void testOOBonInputs() {
        // In1
        assertFalse(dataSink.receivedData.get(3).valid);
        assertEquals(dataSink.receivedData.get(3).value, AdderDefaults.OOB);
        // In2
        assertFalse(dataSink.receivedData.get(4).valid);
        assertEquals(dataSink.receivedData.get(4).value, AdderDefaults.OOB);
    }
    
    public void testLASTonInputs() {
        // first LAST on in2
        assertTrue(dataSink.receivedData.get(5).valid);
        assertEquals(dataSink.receivedData.get(5).value, 3);
        
        assertFalse(dataSink.receivedData.get(6).valid);
        assertNull(dataSink.receivedData.get(6).value);        
        
        assertTrue(dataSink.receivedData.get(7).valid);
        assertEquals(dataSink.receivedData.get(7).value, 3);

        // first LAST on in1
        assertFalse(dataSink.receivedData.get(8).valid);
        assertNull(dataSink.receivedData.get(8).value);        
        
        assertTrue(dataSink.receivedData.get(9).valid);
        assertEquals(dataSink.receivedData.get(9).value, 3);
    }
    
    public void testFinish() {
        // test whether no more outputs generated than required
        assertEquals(dataSink.receivedData.size(), 10);
    }
}
